'use strict';

var jsxRuntime = require('react/jsx-runtime');
var LexicalComposerContext = require('@lexical/react/LexicalComposerContext');
var useLexicalNodeSelection = require('@lexical/react/useLexicalNodeSelection');
var require$$4 = require('@lexical/utils');
var require$$0 = require('lexical');
var React = require('react');
var ImageResizer = require('./ImageResizer-991cf892.js');
var index = require('./index-512be54a.js');
var excalidraw = require('@excalidraw/excalidraw');
var ReactDOM = require('react-dom');
require('@lexical/react/LexicalComposer');
require('@lexical/react/LexicalHistoryPlugin');
require('@lexical/react/LexicalAutoFocusPlugin');
require('@lexical/react/LexicalCharacterLimitPlugin');
require('@lexical/react/LexicalCheckListPlugin');
require('@lexical/react/LexicalClearEditorPlugin');
require('@lexical/react/LexicalClickableLinkPlugin');
require('@lexical/react/LexicalErrorBoundary');
require('@lexical/react/LexicalHashtagPlugin');
require('@lexical/react/LexicalHorizontalRulePlugin');
require('@lexical/react/LexicalListPlugin');
require('@lexical/react/LexicalPlainTextPlugin');
require('@lexical/react/LexicalRichTextPlugin');
require('@lexical/react/LexicalTabIndentationPlugin');
require('@lexical/react/LexicalTablePlugin');
require('@lexical/react/useLexicalEditable');
require('@lexical/code');
require('@lexical/hashtag');
require('@lexical/link');
require('@lexical/list');
require('@lexical/rich-text');
require('@lexical/selection');
require('katex');
require('@lexical/react/LexicalCollaborationContext');
require('yjs');
require('@lexical/react/LexicalHorizontalRuleNode');
require('@lexical/table');
require('@lexical/react/LexicalBlockWithAlignableContents');
require('@lexical/react/LexicalDecoratorBlockNode');
require('@lexical/react/LexicalAutoEmbedPlugin');
require('@lexical/react/LexicalAutoLinkPlugin');
require('lodash-es');
require('@lexical/react/LexicalTypeaheadMenuPlugin');
require('katex/dist/katex.css');
require('react-error-boundary');
require('react-loading-skeleton');
require('@cpchain-foundation/ui');
require('@lexical/react/LexicalContextMenuPlugin');
require('@lexical/mark');
require('@lexical/react/LexicalEditorRefPlugin');
require('@lexical/react/LexicalOnChangePlugin');
require('@lexical/react/LexicalContentEditable');
require('@lexical/react/useLexicalTextEntity');
require('@lexical/react/LexicalLinkPlugin');
require('@lexical/react/LexicalMarkdownShortcutPlugin');
require('@lexical/react/LexicalTableOfContents');
require('@lexical/react/LexicalTreeView');
require('@lexical/overflow');

// exportToSvg has fonts from excalidraw.com
// We don't want them to be used in open source
const removeStyleFromSvg_HACK = (svg) => {
    var _a;
    const styleTag = (_a = svg === null || svg === void 0 ? void 0 : svg.firstElementChild) === null || _a === void 0 ? void 0 : _a.firstElementChild;
    // Generated SVG is getting double-sized by height and width attributes
    // We want to match the real size of the SVG element
    const viewBox = svg.getAttribute('viewBox');
    if (viewBox != null) {
        const viewBoxDimensions = viewBox.split(' ');
        svg.setAttribute('width', viewBoxDimensions[2]);
        svg.setAttribute('height', viewBoxDimensions[3]);
    }
    if (styleTag && styleTag.tagName === 'style') {
        styleTag.remove();
    }
};
/**
 * @explorer-desc
 * A component for rendering Excalidraw elements as a static image
 */
function ExcalidrawImage({ elements, files, imageContainerRef, appState, rootClassName = null, }) {
    var _a;
    const [Svg, setSvg] = React.useState(null);
    React.useEffect(() => {
        const setContent = () => index.__awaiter(this, void 0, void 0, function* () {
            const svg = yield excalidraw.exportToSvg({
                appState,
                elements,
                files,
            });
            removeStyleFromSvg_HACK(svg);
            svg.setAttribute('width', '100%');
            svg.setAttribute('height', '100%');
            svg.setAttribute('display', 'block');
            setSvg(svg);
        });
        setContent();
    }, [elements, files, appState]);
    return (jsxRuntime.jsx("div", { ref: imageContainerRef, className: rootClassName !== null && rootClassName !== void 0 ? rootClassName : '', dangerouslySetInnerHTML: { __html: (_a = Svg === null || Svg === void 0 ? void 0 : Svg.outerHTML) !== null && _a !== void 0 ? _a : '' } }));
}

var css_248z = "/**\n * Copyright (c) Meta Platforms, Inc. and affiliates.\n *\n * This source code is licensed under the MIT license found in the\n * LICENSE file in the root directory of this source tree.\n *\n *\n */\n\n.ExcalidrawModal__overlay {\n  display: flex;\n  align-items: center;\n  position: fixed;\n  flex-direction: column;\n  top: 0px;\n  bottom: 0px;\n  left: 0px;\n  right: 0px;\n  flex-grow: 0px;\n  flex-shrink: 1px;\n  z-index: 100;\n  background-color: rgba(40, 40, 40, 0.6);\n}\n.ExcalidrawModal__actions {\n  text-align: end;\n  position: absolute;\n  right: 5px;\n  top: 5px;\n  z-index: 1;\n}\n.ExcalidrawModal__actions button {\n  background-color: #fff;\n  border-radius: 5px;\n}\n.ExcalidrawModal__row {\n  position: relative;\n  padding: 40px 5px 5px;\n  width: 70vw;\n  height: 70vh;\n  border-radius: 8px;\n  box-shadow: 0 12px 28px 0 rgba(0, 0, 0, 0.2), 0 2px 4px 0 rgba(0, 0, 0, 0.1),\n    inset 0 0 0 1px rgba(255, 255, 255, 0.5);\n}\n.ExcalidrawModal__row > div {\n  border-radius: 5px;\n}\n.ExcalidrawModal__modal {\n  position: relative;\n  z-index: 10;\n  top: 50px;\n  width: auto;\n  left: 0;\n  display: flex;\n  justify-content: center;\n  align-items: center;\n  border-radius: 8px;\n  background-color: #eee;\n}\n.ExcalidrawModal__discardModal {\n  margin-top: 60px;\n  text-align: center;\n}\n";
index.styleInject(css_248z);

/**
 * @explorer-desc
 * A component which renders a modal with Excalidraw (a painting app)
 * which can be used to export an editable image
 */
function ExcalidrawModal({ closeOnClickOutside = false, onSave, initialElements, initialAppState, initialFiles, isShown = false, onDelete, onClose, }) {
    const excaliDrawModelRef = React.useRef(null);
    const excaliDrawSceneRef = React.useRef(null);
    const [discardModalOpen, setDiscardModalOpen] = React.useState(false);
    const [elements, setElements] = React.useState(initialElements);
    const [files, setFiles] = React.useState(initialFiles);
    React.useEffect(() => {
        if (excaliDrawModelRef.current !== null) {
            excaliDrawModelRef.current.focus();
        }
    }, []);
    React.useEffect(() => {
        var _a;
        let modalOverlayElement = null;
        const clickOutsideHandler = (event) => {
            const target = event.target;
            if (excaliDrawModelRef.current !== null &&
                !excaliDrawModelRef.current.contains(target) &&
                closeOnClickOutside) {
                onDelete();
            }
        };
        if (excaliDrawModelRef.current !== null) {
            modalOverlayElement = (_a = excaliDrawModelRef.current) === null || _a === void 0 ? void 0 : _a.parentElement;
            if (modalOverlayElement !== null) {
                modalOverlayElement === null || modalOverlayElement === void 0 ? void 0 : modalOverlayElement.addEventListener('click', clickOutsideHandler);
            }
        }
        return () => {
            if (modalOverlayElement !== null) {
                modalOverlayElement === null || modalOverlayElement === void 0 ? void 0 : modalOverlayElement.removeEventListener('click', clickOutsideHandler);
            }
        };
    }, [closeOnClickOutside, onDelete]);
    React.useLayoutEffect(() => {
        const currentModalRef = excaliDrawModelRef.current;
        const onKeyDown = (event) => {
            if (event.key === 'Escape') {
                onDelete();
            }
        };
        if (currentModalRef !== null) {
            currentModalRef.addEventListener('keydown', onKeyDown);
        }
        return () => {
            if (currentModalRef !== null) {
                currentModalRef.removeEventListener('keydown', onKeyDown);
            }
        };
    }, [elements, files, onDelete]);
    const save = () => {
        var _a;
        if (elements.filter((el) => !el.isDeleted).length > 0) {
            const appState = (_a = excaliDrawSceneRef === null || excaliDrawSceneRef === void 0 ? void 0 : excaliDrawSceneRef.current) === null || _a === void 0 ? void 0 : _a.getAppState();
            if (appState) {
                // We only need a subset of the state
                const partialState = {
                    exportBackground: appState.exportBackground,
                    exportScale: appState.exportScale,
                    exportWithDarkMode: appState.theme === 'dark',
                    isBindingEnabled: appState.isBindingEnabled,
                    isLoading: appState.isLoading,
                    name: appState.name,
                    theme: appState.theme,
                    viewBackgroundColor: appState.viewBackgroundColor,
                    viewModeEnabled: appState.viewModeEnabled,
                    zenModeEnabled: appState.zenModeEnabled,
                    zoom: appState.zoom,
                };
                onSave(elements, partialState, files);
            }
        }
        else {
            // delete node if the scene is clear
            onDelete();
        }
    };
    const discard = () => {
        if (elements.filter((el) => !el.isDeleted).length === 0) {
            // delete node if the scene is clear
            onDelete();
        }
        else {
            //Otherwise, show confirmation dialog before closing
            setDiscardModalOpen(true);
        }
    };
    function ShowDiscardDialog() {
        return (jsxRuntime.jsxs(index.Modal, { title: "Discard", onClose: () => {
                setDiscardModalOpen(false);
            }, closeOnClickOutside: false, children: ["Are you sure you want to discard the changes?", jsxRuntime.jsxs("div", { className: "ExcalidrawModal__discardModal", children: [jsxRuntime.jsx(index.Button, { onClick: () => {
                                setDiscardModalOpen(false);
                                onClose();
                            }, children: "Discard" }), ' ', jsxRuntime.jsx(index.Button, { onClick: () => {
                                setDiscardModalOpen(false);
                            }, children: "Cancel" })] })] }));
    }
    if (isShown === false) {
        return null;
    }
    const onChange = (els, _, fls) => {
        setElements(els);
        setFiles(fls);
    };
    // This is a hacky work-around for Excalidraw + Vite.
    // In DEV, Vite pulls this in fine, in prod it doesn't. It seems
    // like a module resolution issue with ESM vs CJS?
    const _Excalidraw = excalidraw.Excalidraw.$$typeof != null ? excalidraw.Excalidraw : excalidraw.Excalidraw.default;
    return ReactDOM.createPortal(jsxRuntime.jsx("div", { className: "ExcalidrawModal__overlay", role: "dialog", children: jsxRuntime.jsx("div", { className: "ExcalidrawModal__modal", ref: excaliDrawModelRef, tabIndex: -1, children: jsxRuntime.jsxs("div", { className: "ExcalidrawModal__row", children: [discardModalOpen && jsxRuntime.jsx(ShowDiscardDialog, {}), jsxRuntime.jsx(_Excalidraw, { onChange: onChange, ref: excaliDrawSceneRef, initialData: {
                            appState: initialAppState || { isLoading: false },
                            elements: initialElements,
                            files: initialFiles,
                        } }), jsxRuntime.jsxs("div", { className: "ExcalidrawModal__actions", children: [jsxRuntime.jsx("button", { className: "action-button", onClick: discard, children: "Discard" }), jsxRuntime.jsx("button", { className: "action-button", onClick: save, children: "Save" })] })] }) }) }), document.body);
}

function ExcalidrawComponent({ nodeKey, data, }) {
    const [editor] = LexicalComposerContext.useLexicalComposerContext();
    const [isModalOpen, setModalOpen] = React.useState(data === '[]' && editor.isEditable());
    const imageContainerRef = React.useRef(null);
    const buttonRef = React.useRef(null);
    const captionButtonRef = React.useRef(null);
    const [isSelected, setSelected, clearSelection] = useLexicalNodeSelection.useLexicalNodeSelection(nodeKey);
    const [isResizing, setIsResizing] = React.useState(false);
    const onDelete = React.useCallback((event) => {
        if (isSelected && require$$0.$isNodeSelection(require$$0.$getSelection())) {
            event.preventDefault();
            editor.update(() => {
                const node = require$$0.$getNodeByKey(nodeKey);
                if (index.$isExcalidrawNode(node)) {
                    node.remove();
                }
            });
        }
        return false;
    }, [editor, isSelected, nodeKey]);
    // Set editor to readOnly if excalidraw is open to prevent unwanted changes
    React.useEffect(() => {
        if (isModalOpen) {
            editor.setEditable(false);
        }
        else {
            editor.setEditable(true);
        }
    }, [isModalOpen, editor]);
    React.useEffect(() => {
        return require$$4.mergeRegister(editor.registerCommand(require$$0.CLICK_COMMAND, (event) => {
            const buttonElem = buttonRef.current;
            const eventTarget = event.target;
            if (isResizing) {
                return true;
            }
            if (buttonElem !== null && buttonElem.contains(eventTarget)) {
                if (!event.shiftKey) {
                    clearSelection();
                }
                setSelected(!isSelected);
                if (event.detail > 1) {
                    setModalOpen(true);
                }
                return true;
            }
            return false;
        }, require$$0.COMMAND_PRIORITY_LOW), editor.registerCommand(require$$0.KEY_DELETE_COMMAND, onDelete, require$$0.COMMAND_PRIORITY_LOW), editor.registerCommand(require$$0.KEY_BACKSPACE_COMMAND, onDelete, require$$0.COMMAND_PRIORITY_LOW));
    }, [clearSelection, editor, isSelected, isResizing, onDelete, setSelected]);
    const deleteNode = React.useCallback(() => {
        setModalOpen(false);
        return editor.update(() => {
            const node = require$$0.$getNodeByKey(nodeKey);
            if (index.$isExcalidrawNode(node)) {
                node.remove();
            }
        });
    }, [editor, nodeKey]);
    const setData = (els, aps, fls) => {
        if (!editor.isEditable()) {
            return;
        }
        return editor.update(() => {
            const node = require$$0.$getNodeByKey(nodeKey);
            if (index.$isExcalidrawNode(node)) {
                if (els.length > 0 || Object.keys(fls).length > 0) {
                    node.setData(JSON.stringify({
                        appState: aps,
                        elements: els,
                        files: fls,
                    }));
                }
                else {
                    node.remove();
                }
            }
        });
    };
    const onResizeStart = () => {
        setIsResizing(true);
    };
    const onResizeEnd = () => {
        // Delay hiding the resize bars for click case
        setTimeout(() => {
            setIsResizing(false);
        }, 200);
    };
    const openModal = React.useCallback(() => {
        setModalOpen(true);
    }, []);
    const { elements = [], files = {}, appState = {}, } = React.useMemo(() => JSON.parse(data), [data]);
    return (jsxRuntime.jsxs(jsxRuntime.Fragment, { children: [jsxRuntime.jsx(ExcalidrawModal, { initialElements: elements, initialFiles: files, initialAppState: appState, isShown: isModalOpen, onDelete: deleteNode, onClose: () => setModalOpen(false), onSave: (els, aps, fls) => {
                    editor.setEditable(true);
                    setData(els, aps, fls);
                    setModalOpen(false);
                }, closeOnClickOutside: false }), elements.length > 0 && (jsxRuntime.jsxs("button", { ref: buttonRef, className: `excalidraw-button ${isSelected ? 'selected' : ''}`, children: [jsxRuntime.jsx(ExcalidrawImage, { imageContainerRef: imageContainerRef, className: "image", elements: elements, files: files, appState: appState }), isSelected && (jsxRuntime.jsx("div", { className: "image-edit-button", role: "button", tabIndex: 0, onMouseDown: (event) => event.preventDefault(), onClick: openModal })), (isSelected || isResizing) && (jsxRuntime.jsx(ImageResizer.ImageResizer, { buttonRef: captionButtonRef, showCaption: true, setShowCaption: () => null, imageRef: imageContainerRef, editor: editor, onResizeStart: onResizeStart, onResizeEnd: onResizeEnd, captionsEnabled: true }))] }))] }));
}

exports["default"] = ExcalidrawComponent;
