'use strict';

var index = require('./index-512be54a.js');
var jsxRuntime = require('react/jsx-runtime');
var clipboard = require('@lexical/clipboard');
var LexicalComposerContext = require('@lexical/react/LexicalComposerContext');
var LexicalNestedComposer = require('@lexical/react/LexicalNestedComposer');
var useLexicalNodeSelection = require('@lexical/react/useLexicalNodeSelection');
var require$$4 = require('@lexical/utils');
var require$$0 = require('lexical');
var React = require('react');
var ReactDOM = require('react-dom');
require('@lexical/react/LexicalComposer');
require('@lexical/react/LexicalHistoryPlugin');
require('@lexical/react/LexicalAutoFocusPlugin');
require('@lexical/react/LexicalCharacterLimitPlugin');
require('@lexical/react/LexicalCheckListPlugin');
require('@lexical/react/LexicalClearEditorPlugin');
require('@lexical/react/LexicalClickableLinkPlugin');
require('@lexical/react/LexicalErrorBoundary');
require('@lexical/react/LexicalHashtagPlugin');
require('@lexical/react/LexicalHorizontalRulePlugin');
require('@lexical/react/LexicalListPlugin');
require('@lexical/react/LexicalPlainTextPlugin');
require('@lexical/react/LexicalRichTextPlugin');
require('@lexical/react/LexicalTabIndentationPlugin');
require('@lexical/react/LexicalTablePlugin');
require('@lexical/react/useLexicalEditable');
require('@lexical/code');
require('@lexical/hashtag');
require('@lexical/link');
require('@lexical/list');
require('@lexical/rich-text');
require('@lexical/selection');
require('katex');
require('@lexical/react/LexicalCollaborationContext');
require('yjs');
require('@lexical/react/LexicalHorizontalRuleNode');
require('@lexical/table');
require('@lexical/react/LexicalBlockWithAlignableContents');
require('@lexical/react/LexicalDecoratorBlockNode');
require('@lexical/react/LexicalAutoEmbedPlugin');
require('@lexical/react/LexicalAutoLinkPlugin');
require('lodash-es');
require('@lexical/react/LexicalTypeaheadMenuPlugin');
require('katex/dist/katex.css');
require('react-error-boundary');
require('react-loading-skeleton');
require('@cpchain-foundation/ui');
require('@lexical/react/LexicalContextMenuPlugin');
require('@lexical/mark');
require('@lexical/react/LexicalEditorRefPlugin');
require('@lexical/react/LexicalOnChangePlugin');
require('@lexical/react/LexicalContentEditable');
require('@lexical/react/useLexicalTextEntity');
require('@lexical/react/LexicalLinkPlugin');
require('@lexical/react/LexicalMarkdownShortcutPlugin');
require('@lexical/react/LexicalTableOfContents');
require('@lexical/react/LexicalTreeView');
require('@lexical/overflow');

const NO_CELLS = [];
function $createSelectAll() {
    const sel = require$$0.$createRangeSelection();
    sel.focus.set('root', require$$0.$getRoot().getChildrenSize(), 'element');
    return sel;
}
function createEmptyParagraphHTML(theme) {
    return `<p class="${theme.paragraph}"><br></p>`;
}
function focusCell(tableElem, id) {
    const cellElem = tableElem.querySelector(`[data-id=${id}]`);
    if (cellElem == null) {
        return;
    }
    cellElem.focus();
}
function isStartingResize(target) {
    return target.nodeType === 1 && target.hasAttribute('data-table-resize');
}
function generateHTMLFromJSON(editorStateJSON, cellEditor) {
    const editorState = cellEditor.parseEditorState(editorStateJSON);
    let html = index.cellHTMLCache.get(editorStateJSON);
    if (html === undefined) {
        html = editorState.read(() => index.LexicalHtml_1.$generateHtmlFromNodes(cellEditor, null));
        const textContent = editorState.read(() => require$$0.$getRoot().getTextContent());
        index.cellHTMLCache.set(editorStateJSON, html);
        index.cellTextContentCache.set(editorStateJSON, textContent);
    }
    return html;
}
function getCurrentDocument(editor) {
    const rootElement = editor.getRootElement();
    return rootElement !== null ? rootElement.ownerDocument : document;
}
function isCopy(keyCode, shiftKey, metaKey, ctrlKey) {
    if (shiftKey) {
        return false;
    }
    if (keyCode === 67) {
        return index.IS_APPLE ? metaKey : ctrlKey;
    }
    return false;
}
function isCut(keyCode, shiftKey, metaKey, ctrlKey) {
    if (shiftKey) {
        return false;
    }
    if (keyCode === 88) {
        return index.IS_APPLE ? metaKey : ctrlKey;
    }
    return false;
}
function isPaste(keyCode, shiftKey, metaKey, ctrlKey) {
    if (shiftKey) {
        return false;
    }
    if (keyCode === 86) {
        return index.IS_APPLE ? metaKey : ctrlKey;
    }
    return false;
}
function getCellID(domElement) {
    let node = domElement;
    while (node !== null) {
        const possibleID = node.getAttribute('data-id');
        if (possibleID != null) {
            return possibleID;
        }
        node = node.parentElement;
    }
    return null;
}
function getTableCellWidth(domElement) {
    let node = domElement;
    while (node !== null) {
        if (node.nodeName === 'TH' || node.nodeName === 'TD') {
            return node.getBoundingClientRect().width;
        }
        node = node.parentElement;
    }
    return 0;
}
function $updateCells(rows, ids, cellCoordMap, cellEditor, updateTableNode, fn) {
    for (const id of ids) {
        const cell = getCell(rows, id, cellCoordMap);
        if (cell !== null && cellEditor !== null) {
            const editorState = cellEditor.parseEditorState(cell.json);
            cellEditor._headless = true;
            cellEditor.setEditorState(editorState);
            cellEditor.update(fn, { discrete: true });
            cellEditor._headless = false;
            const newJSON = JSON.stringify(cellEditor.getEditorState());
            updateTableNode((tableNode) => {
                const [x, y] = cellCoordMap.get(id);
                require$$0.$addUpdateTag('history-push');
                tableNode.updateCellJSON(x, y, newJSON);
            });
        }
    }
}
function isTargetOnPossibleUIControl(target) {
    let node = target;
    while (node !== null) {
        const nodeName = node.nodeName;
        if (nodeName === 'BUTTON' ||
            nodeName === 'INPUT' ||
            nodeName === 'TEXTAREA') {
            return true;
        }
        node = node.parentElement;
    }
    return false;
}
function getSelectedRect(startID, endID, cellCoordMap) {
    const startCoords = cellCoordMap.get(startID);
    const endCoords = cellCoordMap.get(endID);
    if (startCoords === undefined || endCoords === undefined) {
        return null;
    }
    const startX = Math.min(startCoords[0], endCoords[0]);
    const endX = Math.max(startCoords[0], endCoords[0]);
    const startY = Math.min(startCoords[1], endCoords[1]);
    const endY = Math.max(startCoords[1], endCoords[1]);
    return {
        endX,
        endY,
        startX,
        startY,
    };
}
function getSelectedIDs(rows, startID, endID, cellCoordMap) {
    const rect = getSelectedRect(startID, endID, cellCoordMap);
    if (rect === null) {
        return [];
    }
    const { startX, endY, endX, startY } = rect;
    const ids = [];
    for (let x = startX; x <= endX; x++) {
        for (let y = startY; y <= endY; y++) {
            ids.push(rows[y].cells[x].id);
        }
    }
    return ids;
}
function extractCellsFromRows(rows, rect) {
    const { startX, endY, endX, startY } = rect;
    const newRows = [];
    for (let y = startY; y <= endY; y++) {
        const row = rows[y];
        const newRow = index.createRow();
        for (let x = startX; x <= endX; x++) {
            const cellClone = Object.assign({}, row.cells[x]);
            cellClone.id = index.createUID();
            newRow.cells.push(cellClone);
        }
        newRows.push(newRow);
    }
    return newRows;
}
function TableCellEditor({ cellEditor }) {
    const { cellEditorConfig, cellEditorPlugins } = React.useContext(index.CellContext);
    if (cellEditorPlugins === null || cellEditorConfig === null) {
        return null;
    }
    return (jsxRuntime.jsx(LexicalNestedComposer.LexicalNestedComposer, { initialEditor: cellEditor, initialTheme: cellEditorConfig.theme, initialNodes: cellEditorConfig.nodes, skipCollabChecks: true, children: cellEditorPlugins }));
}
function getCell(rows, cellID, cellCoordMap) {
    const coords = cellCoordMap.get(cellID);
    if (coords === undefined) {
        return null;
    }
    const [x, y] = coords;
    const row = rows[y];
    return row.cells[x];
}
function TableActionMenu({ cell, rows, cellCoordMap, menuElem, updateCellsByID, onClose, updateTableNode, setSortingOptions, sortingOptions, }) {
    const dropDownRef = React.useRef(null);
    React.useEffect(() => {
        const dropdownElem = dropDownRef.current;
        if (dropdownElem !== null) {
            const rect = menuElem.getBoundingClientRect();
            dropdownElem.style.top = `${rect.y}px`;
            dropdownElem.style.left = `${rect.x}px`;
        }
    }, [menuElem]);
    React.useEffect(() => {
        const handleClickOutside = (event) => {
            const dropdownElem = dropDownRef.current;
            if (dropdownElem !== null &&
                !dropdownElem.contains(event.target)) {
                event.stopPropagation();
            }
        };
        window.addEventListener('click', handleClickOutside);
        return () => window.removeEventListener('click', handleClickOutside);
    }, [onClose]);
    const coords = cellCoordMap.get(cell.id);
    if (coords === undefined) {
        return null;
    }
    const [x, y] = coords;
    return (
    // eslint-disable-next-line jsx-a11y/no-static-element-interactions
    jsxRuntime.jsxs("div", { className: "dropdown", ref: dropDownRef, onPointerMove: (e) => {
            e.stopPropagation();
        }, onPointerDown: (e) => {
            e.stopPropagation();
        }, onPointerUp: (e) => {
            e.stopPropagation();
        }, onClick: (e) => {
            e.stopPropagation();
        }, children: [jsxRuntime.jsx("button", { className: "item", onClick: () => {
                    updateTableNode((tableNode) => {
                        require$$0.$addUpdateTag('history-push');
                        tableNode.updateCellType(x, y, cell.type === 'normal' ? 'header' : 'normal');
                    });
                    onClose();
                }, children: jsxRuntime.jsx("span", { className: "text", children: cell.type === 'normal' ? 'Make header' : 'Remove header' }) }), jsxRuntime.jsx("button", { className: "item", onClick: () => {
                    updateCellsByID([cell.id], () => {
                        const root = require$$0.$getRoot();
                        root.clear();
                        root.append(require$$0.$createParagraphNode());
                    });
                    onClose();
                }, children: jsxRuntime.jsx("span", { className: "text", children: "Clear cell" }) }), jsxRuntime.jsx("hr", {}), cell.type === 'header' && y === 0 && (jsxRuntime.jsxs(jsxRuntime.Fragment, { children: [sortingOptions !== null && sortingOptions.x === x && (jsxRuntime.jsx("button", { className: "item", onClick: () => {
                            setSortingOptions(null);
                            onClose();
                        }, children: jsxRuntime.jsx("span", { className: "text", children: "Remove sorting" }) })), (sortingOptions === null ||
                        sortingOptions.x !== x ||
                        sortingOptions.type === 'descending') && (jsxRuntime.jsx("button", { className: "item", onClick: () => {
                            setSortingOptions({ type: 'ascending', x });
                            onClose();
                        }, children: jsxRuntime.jsx("span", { className: "text", children: "Sort ascending" }) })), (sortingOptions === null ||
                        sortingOptions.x !== x ||
                        sortingOptions.type === 'ascending') && (jsxRuntime.jsx("button", { className: "item", onClick: () => {
                            setSortingOptions({ type: 'descending', x });
                            onClose();
                        }, children: jsxRuntime.jsx("span", { className: "text", children: "Sort descending" }) })), jsxRuntime.jsx("hr", {})] })), jsxRuntime.jsx("button", { className: "item", onClick: () => {
                    updateTableNode((tableNode) => {
                        require$$0.$addUpdateTag('history-push');
                        tableNode.insertRowAt(y);
                    });
                    onClose();
                }, children: jsxRuntime.jsx("span", { className: "text", children: "Insert row above" }) }), jsxRuntime.jsx("button", { className: "item", onClick: () => {
                    updateTableNode((tableNode) => {
                        require$$0.$addUpdateTag('history-push');
                        tableNode.insertRowAt(y + 1);
                    });
                    onClose();
                }, children: jsxRuntime.jsx("span", { className: "text", children: "Insert row below" }) }), jsxRuntime.jsx("hr", {}), jsxRuntime.jsx("button", { className: "item", onClick: () => {
                    updateTableNode((tableNode) => {
                        require$$0.$addUpdateTag('history-push');
                        tableNode.insertColumnAt(x);
                    });
                    onClose();
                }, children: jsxRuntime.jsx("span", { className: "text", children: "Insert column left" }) }), jsxRuntime.jsx("button", { className: "item", onClick: () => {
                    updateTableNode((tableNode) => {
                        require$$0.$addUpdateTag('history-push');
                        tableNode.insertColumnAt(x + 1);
                    });
                    onClose();
                }, children: jsxRuntime.jsx("span", { className: "text", children: "Insert column right" }) }), jsxRuntime.jsx("hr", {}), rows[0].cells.length !== 1 && (jsxRuntime.jsx("button", { className: "item", onClick: () => {
                    updateTableNode((tableNode) => {
                        require$$0.$addUpdateTag('history-push');
                        tableNode.deleteColumnAt(x);
                    });
                    onClose();
                }, children: jsxRuntime.jsx("span", { className: "text", children: "Delete column" }) })), rows.length !== 1 && (jsxRuntime.jsx("button", { className: "item", onClick: () => {
                    updateTableNode((tableNode) => {
                        require$$0.$addUpdateTag('history-push');
                        tableNode.deleteRowAt(y);
                    });
                    onClose();
                }, children: jsxRuntime.jsx("span", { className: "text", children: "Delete row" }) })), jsxRuntime.jsx("button", { className: "item", onClick: () => {
                    updateTableNode((tableNode) => {
                        require$$0.$addUpdateTag('history-push');
                        tableNode.selectNext();
                        tableNode.remove();
                    });
                    onClose();
                }, children: jsxRuntime.jsx("span", { className: "text", children: "Delete table" }) })] }));
}
function TableCell({ cell, cellCoordMap, cellEditor, isEditing, isSelected, isPrimarySelected, theme, updateCellsByID, updateTableNode, rows, setSortingOptions, sortingOptions, }) {
    const [showMenu, setShowMenu] = React.useState(false);
    const menuRootRef = React.useRef(null);
    const isHeader = cell.type !== 'normal';
    const editorStateJSON = cell.json;
    const CellComponent = isHeader ? 'th' : 'td';
    const cellWidth = cell.width;
    const menuElem = menuRootRef.current;
    const coords = cellCoordMap.get(cell.id);
    const isSorted = sortingOptions !== null &&
        coords !== undefined &&
        coords[0] === sortingOptions.x &&
        coords[1] === 0;
    React.useEffect(() => {
        if (isEditing || !isPrimarySelected) {
            setShowMenu(false);
        }
    }, [isEditing, isPrimarySelected]);
    return (jsxRuntime.jsxs(CellComponent, { className: `${theme.tableCell} ${isHeader ? theme.tableCellHeader : ''} ${isSelected ? theme.tableCellSelected : ''}`, "data-id": cell.id, tabIndex: -1, style: { width: cellWidth !== null ? cellWidth : undefined }, children: [isPrimarySelected && (jsxRuntime.jsx("div", { className: `${theme.tableCellPrimarySelected} ${isEditing ? theme.tableCellEditing : ''}` })), isPrimarySelected && isEditing ? (jsxRuntime.jsx(TableCellEditor, { cellEditor: cellEditor })) : (jsxRuntime.jsxs(jsxRuntime.Fragment, { children: [jsxRuntime.jsx("div", { dangerouslySetInnerHTML: {
                            __html: editorStateJSON === ''
                                ? createEmptyParagraphHTML(theme)
                                : generateHTMLFromJSON(editorStateJSON, cellEditor),
                        } }), jsxRuntime.jsx("div", { className: theme.tableCellResizer, "data-table-resize": "true" })] })), isPrimarySelected && !isEditing && (jsxRuntime.jsx("div", { className: theme.tableCellActionButtonContainer, ref: menuRootRef, children: jsxRuntime.jsx("button", { className: theme.tableCellActionButton, onClick: (e) => {
                        setShowMenu(!showMenu);
                        e.stopPropagation();
                    }, children: jsxRuntime.jsx("i", { className: "chevron-down" }) }) })), showMenu &&
                menuElem !== null &&
                ReactDOM.createPortal(jsxRuntime.jsx(TableActionMenu, { cell: cell, menuElem: menuElem, updateCellsByID: updateCellsByID, onClose: () => setShowMenu(false), updateTableNode: updateTableNode, cellCoordMap: cellCoordMap, rows: rows, setSortingOptions: setSortingOptions, sortingOptions: sortingOptions }), document.body), isSorted && jsxRuntime.jsx("div", { className: theme.tableCellSortedIndicator })] }));
}
function TableComponent({ nodeKey, rows: rawRows, theme, }) {
    const [isSelected, setSelected, clearSelection] = useLexicalNodeSelection.useLexicalNodeSelection(nodeKey);
    const resizeMeasureRef = React.useRef({
        point: 0,
        size: 0,
    });
    const [sortingOptions, setSortingOptions] = React.useState(null);
    const addRowsRef = React.useRef(null);
    const lastCellIDRef = React.useRef(null);
    const tableResizerRulerRef = React.useRef(null);
    const { cellEditorConfig } = React.useContext(index.CellContext);
    const [isEditing, setIsEditing] = React.useState(false);
    const [showAddColumns, setShowAddColumns] = React.useState(false);
    const [showAddRows, setShowAddRows] = React.useState(false);
    const [editor] = LexicalComposerContext.useLexicalComposerContext();
    const mouseDownRef = React.useRef(false);
    const [resizingID, setResizingID] = React.useState(null);
    const tableRef = React.useRef(null);
    const cellCoordMap = React.useMemo(() => {
        const map = new Map();
        for (let y = 0; y < rawRows.length; y++) {
            const row = rawRows[y];
            const cells = row.cells;
            for (let x = 0; x < cells.length; x++) {
                const cell = cells[x];
                map.set(cell.id, [x, y]);
            }
        }
        return map;
    }, [rawRows]);
    const rows = React.useMemo(() => {
        if (sortingOptions === null) {
            return rawRows;
        }
        const _rows = rawRows.slice(1);
        _rows.sort((a, b) => {
            const aCells = a.cells;
            const bCells = b.cells;
            const x = sortingOptions.x;
            const aContent = index.cellTextContentCache.get(aCells[x].json) || '';
            const bContent = index.cellTextContentCache.get(bCells[x].json) || '';
            if (aContent === '' || bContent === '') {
                return 1;
            }
            if (sortingOptions.type === 'ascending') {
                return aContent.localeCompare(bContent);
            }
            return bContent.localeCompare(aContent);
        });
        _rows.unshift(rawRows[0]);
        return _rows;
    }, [rawRows, sortingOptions]);
    const [primarySelectedCellID, setPrimarySelectedCellID] = React.useState(null);
    const cellEditor = React.useMemo(() => {
        if (cellEditorConfig === null) {
            return null;
        }
        const _cellEditor = require$$0.createEditor({
            namespace: cellEditorConfig.namespace,
            nodes: cellEditorConfig.nodes,
            onError: (error) => cellEditorConfig.onError(error, _cellEditor),
            theme: cellEditorConfig.theme,
        });
        return _cellEditor;
    }, [cellEditorConfig]);
    const [selectedCellIDs, setSelectedCellIDs] = React.useState([]);
    const selectedCellSet = React.useMemo(() => new Set(selectedCellIDs), [selectedCellIDs]);
    React.useEffect(() => {
        const tableElem = tableRef.current;
        if (isSelected &&
            document.activeElement === document.body &&
            tableElem !== null) {
            tableElem.focus();
        }
    }, [isSelected]);
    const updateTableNode = React.useCallback((fn) => {
        editor.update(() => {
            const tableNode = require$$0.$getNodeByKey(nodeKey);
            if (index.$isTableNode(tableNode)) {
                fn(tableNode);
            }
        });
    }, [editor, nodeKey]);
    const addColumns = () => {
        updateTableNode((tableNode) => {
            require$$0.$addUpdateTag('history-push');
            tableNode.addColumns(1);
        });
    };
    const addRows = () => {
        updateTableNode((tableNode) => {
            require$$0.$addUpdateTag('history-push');
            tableNode.addRows(1);
        });
    };
    const modifySelectedCells = React.useCallback((x, y, extend) => {
        const id = rows[y].cells[x].id;
        lastCellIDRef.current = id;
        if (extend) {
            const selectedIDs = getSelectedIDs(rows, primarySelectedCellID, id, cellCoordMap);
            setSelectedCellIDs(selectedIDs);
        }
        else {
            setPrimarySelectedCellID(id);
            setSelectedCellIDs(NO_CELLS);
            focusCell(tableRef.current, id);
        }
    }, [cellCoordMap, primarySelectedCellID, rows]);
    const saveEditorToJSON = React.useCallback(() => {
        if (cellEditor !== null && primarySelectedCellID !== null) {
            const json = JSON.stringify(cellEditor.getEditorState());
            updateTableNode((tableNode) => {
                const coords = cellCoordMap.get(primarySelectedCellID);
                if (coords === undefined) {
                    return;
                }
                require$$0.$addUpdateTag('history-push');
                const [x, y] = coords;
                tableNode.updateCellJSON(x, y, json);
            });
        }
    }, [cellCoordMap, cellEditor, primarySelectedCellID, updateTableNode]);
    const selectTable = React.useCallback(() => {
        setTimeout(() => {
            var _a;
            const parentRootElement = editor.getRootElement();
            if (parentRootElement !== null) {
                parentRootElement.focus({ preventScroll: true });
                (_a = window.getSelection()) === null || _a === void 0 ? void 0 : _a.removeAllRanges();
            }
        }, 20);
    }, [editor]);
    React.useEffect(() => {
        const tableElem = tableRef.current;
        if (tableElem === null) {
            return;
        }
        const doc = getCurrentDocument(editor);
        const isAtEdgeOfTable = (event) => {
            const x = event.clientX - tableRect.x;
            const y = event.clientY - tableRect.y;
            return x < 5 || y < 5;
        };
        const handlePointerDown = (event) => {
            const possibleID = getCellID(event.target);
            if (possibleID !== null &&
                editor.isEditable() &&
                tableElem.contains(event.target)) {
                if (isAtEdgeOfTable(event)) {
                    setSelected(true);
                    setPrimarySelectedCellID(null);
                    selectTable();
                    return;
                }
                setSelected(false);
                if (isStartingResize(event.target)) {
                    setResizingID(possibleID);
                    tableElem.style.userSelect = 'none';
                    resizeMeasureRef.current = {
                        point: event.clientX,
                        size: getTableCellWidth(event.target),
                    };
                    return;
                }
                mouseDownRef.current = true;
                if (primarySelectedCellID !== possibleID) {
                    if (isEditing) {
                        saveEditorToJSON();
                    }
                    setPrimarySelectedCellID(possibleID);
                    setIsEditing(false);
                    lastCellIDRef.current = possibleID;
                }
                else {
                    lastCellIDRef.current = null;
                }
                setSelectedCellIDs(NO_CELLS);
            }
            else if (primarySelectedCellID !== null &&
                !isTargetOnPossibleUIControl(event.target)) {
                setSelected(false);
                mouseDownRef.current = false;
                if (isEditing) {
                    saveEditorToJSON();
                }
                setPrimarySelectedCellID(null);
                setSelectedCellIDs(NO_CELLS);
                setIsEditing(false);
                lastCellIDRef.current = null;
            }
        };
        const tableRect = tableElem.getBoundingClientRect();
        const handlePointerMove = (event) => {
            if (resizingID !== null) {
                const tableResizerRulerElem = tableResizerRulerRef.current;
                if (tableResizerRulerElem !== null) {
                    const { size, point } = resizeMeasureRef.current;
                    const diff = event.clientX - point;
                    const newWidth = size + diff;
                    let x = event.clientX - tableRect.x;
                    if (x < 10) {
                        x = 10;
                    }
                    else if (x > tableRect.width - 10) {
                        x = tableRect.width - 10;
                    }
                    else if (newWidth < 20) {
                        x = point - size + 20 - tableRect.x;
                    }
                    tableResizerRulerElem.style.left = `${x}px`;
                }
                return;
            }
            if (!isEditing) {
                const { clientX, clientY } = event;
                const { width, x, y, height } = tableRect;
                const isOnRightEdge = clientX > x + width * 0.9 &&
                    clientX < x + width + 40 &&
                    !mouseDownRef.current;
                setShowAddColumns(isOnRightEdge);
                const isOnBottomEdge = event.target === addRowsRef.current ||
                    (clientY > y + height * 0.85 &&
                        clientY < y + height + 5 &&
                        !mouseDownRef.current);
                setShowAddRows(isOnBottomEdge);
            }
            if (isEditing ||
                !mouseDownRef.current ||
                primarySelectedCellID === null) {
                return;
            }
            const possibleID = getCellID(event.target);
            if (possibleID !== null && possibleID !== lastCellIDRef.current) {
                if (selectedCellIDs.length === 0) {
                    tableElem.style.userSelect = 'none';
                }
                const selectedIDs = getSelectedIDs(rows, primarySelectedCellID, possibleID, cellCoordMap);
                if (selectedIDs.length === 1) {
                    setSelectedCellIDs(NO_CELLS);
                }
                else {
                    setSelectedCellIDs(selectedIDs);
                }
                lastCellIDRef.current = possibleID;
            }
        };
        const handlePointerUp = (event) => {
            var _a;
            if (resizingID !== null) {
                const { size, point } = resizeMeasureRef.current;
                const diff = event.clientX - point;
                let newWidth = size + diff;
                if (newWidth < 10) {
                    newWidth = 10;
                }
                updateTableNode((tableNode) => {
                    const [x] = cellCoordMap.get(resizingID);
                    require$$0.$addUpdateTag('history-push');
                    tableNode.updateColumnWidth(x, newWidth);
                });
                setResizingID(null);
            }
            if (tableElem !== null &&
                selectedCellIDs.length > 1 &&
                mouseDownRef.current) {
                tableElem.style.userSelect = 'text';
                (_a = window.getSelection()) === null || _a === void 0 ? void 0 : _a.removeAllRanges();
            }
            mouseDownRef.current = false;
        };
        doc.addEventListener('pointerdown', handlePointerDown);
        doc.addEventListener('pointermove', handlePointerMove);
        doc.addEventListener('pointerup', handlePointerUp);
        return () => {
            doc.removeEventListener('pointerdown', handlePointerDown);
            doc.removeEventListener('pointermove', handlePointerMove);
            doc.removeEventListener('pointerup', handlePointerUp);
        };
    }, [
        cellEditor,
        editor,
        isEditing,
        rows,
        saveEditorToJSON,
        primarySelectedCellID,
        selectedCellSet,
        selectedCellIDs,
        cellCoordMap,
        resizingID,
        updateTableNode,
        setSelected,
        selectTable,
    ]);
    React.useEffect(() => {
        if (!isEditing && primarySelectedCellID !== null) {
            const doc = getCurrentDocument(editor);
            const loadContentIntoCell = (cell) => {
                if (cell !== null && cellEditor !== null) {
                    const editorStateJSON = cell.json;
                    const editorState = cellEditor.parseEditorState(editorStateJSON);
                    cellEditor.setEditorState(editorState);
                }
            };
            const handleDblClick = (event) => {
                const possibleID = getCellID(event.target);
                if (possibleID === primarySelectedCellID && editor.isEditable()) {
                    const cell = getCell(rows, possibleID, cellCoordMap);
                    loadContentIntoCell(cell);
                    setIsEditing(true);
                    setSelectedCellIDs(NO_CELLS);
                }
            };
            const handleKeyDown = (event) => {
                // Ignore arrow keys, escape or tab
                const keyCode = event.keyCode;
                if (keyCode === 16 ||
                    keyCode === 27 ||
                    keyCode === 9 ||
                    keyCode === 37 ||
                    keyCode === 38 ||
                    keyCode === 39 ||
                    keyCode === 40 ||
                    keyCode === 8 ||
                    keyCode === 46 ||
                    !editor.isEditable()) {
                    return;
                }
                if (keyCode === 13) {
                    event.preventDefault();
                }
                if (!isEditing &&
                    primarySelectedCellID !== null &&
                    editor.getEditorState().read(() => require$$0.$getSelection() === null) &&
                    event.target.contentEditable !== 'true') {
                    if (isCopy(keyCode, event.shiftKey, event.metaKey, event.ctrlKey)) {
                        editor.dispatchCommand(require$$0.COPY_COMMAND, event);
                        return;
                    }
                    if (isCut(keyCode, event.shiftKey, event.metaKey, event.ctrlKey)) {
                        editor.dispatchCommand(require$$0.CUT_COMMAND, event);
                        return;
                    }
                    if (isPaste(keyCode, event.shiftKey, event.metaKey, event.ctrlKey)) {
                        editor.dispatchCommand(require$$0.PASTE_COMMAND, event);
                        return;
                    }
                }
                if (event.metaKey || event.ctrlKey || event.altKey) {
                    return;
                }
                const cell = getCell(rows, primarySelectedCellID, cellCoordMap);
                loadContentIntoCell(cell);
                setIsEditing(true);
                setSelectedCellIDs(NO_CELLS);
            };
            doc.addEventListener('dblclick', handleDblClick);
            doc.addEventListener('keydown', handleKeyDown);
            return () => {
                doc.removeEventListener('dblclick', handleDblClick);
                doc.removeEventListener('keydown', handleKeyDown);
            };
        }
    }, [
        cellEditor,
        editor,
        isEditing,
        rows,
        primarySelectedCellID,
        cellCoordMap,
    ]);
    const updateCellsByID = React.useCallback((ids, fn) => {
        $updateCells(rows, ids, cellCoordMap, cellEditor, updateTableNode, fn);
    }, [cellCoordMap, cellEditor, rows, updateTableNode]);
    const clearCellsCommand = React.useCallback(() => {
        if (primarySelectedCellID !== null && !isEditing) {
            updateCellsByID([primarySelectedCellID, ...selectedCellIDs], () => {
                const root = require$$0.$getRoot();
                root.clear();
                root.append(require$$0.$createParagraphNode());
            });
            return true;
        }
        else if (isSelected) {
            updateTableNode((tableNode) => {
                require$$0.$addUpdateTag('history-push');
                tableNode.selectNext();
                tableNode.remove();
            });
        }
        return false;
    }, [
        isEditing,
        isSelected,
        primarySelectedCellID,
        selectedCellIDs,
        updateCellsByID,
        updateTableNode,
    ]);
    React.useEffect(() => {
        const tableElem = tableRef.current;
        if (tableElem === null) {
            return;
        }
        const copyDataToClipboard = (event, htmlString, lexicalString, plainTextString) => {
            const clipboardData = event instanceof KeyboardEvent ? null : event.clipboardData;
            event.preventDefault();
            if (clipboardData != null) {
                clipboardData.setData('text/html', htmlString);
                clipboardData.setData('text/plain', plainTextString);
                clipboardData.setData('application/x-lexical-editor', lexicalString);
            }
            else {
                const clipboard = navigator.clipboard;
                if (clipboard != null) {
                    // Most browsers only support a single item in the clipboard at one time.
                    // So we optimize by only putting in HTML.
                    const data = [
                        new ClipboardItem({
                            'text/html': new Blob([htmlString], {
                                type: 'text/html',
                            }),
                        }),
                    ];
                    clipboard.write(data);
                }
            }
        };
        const getTypeFromObject = (clipboardData, type) => index.__awaiter(this, void 0, void 0, function* () {
            try {
                return clipboardData instanceof DataTransfer
                    ? clipboardData.getData(type)
                    : clipboardData instanceof ClipboardItem
                        ? yield (yield clipboardData.getType(type)).text()
                        : '';
            }
            catch (_a) {
                return '';
            }
        });
        const pasteContent = (event) => index.__awaiter(this, void 0, void 0, function* () {
            let clipboardData = (event instanceof InputEvent ? null : event.clipboardData) || null;
            if (primarySelectedCellID !== null && cellEditor !== null) {
                event.preventDefault();
                if (clipboardData === null) {
                    try {
                        const items = yield navigator.clipboard.read();
                        clipboardData = items[0];
                    }
                    catch (_b) {
                        // NO-OP
                    }
                }
                const lexicalString = clipboardData !== null
                    ? yield getTypeFromObject(clipboardData, 'application/x-lexical-editor')
                    : '';
                if (lexicalString) {
                    try {
                        const payload = JSON.parse(lexicalString);
                        if (payload.namespace === editor._config.namespace &&
                            Array.isArray(payload.nodes)) {
                            $updateCells(rows, [primarySelectedCellID], cellCoordMap, cellEditor, updateTableNode, () => {
                                const root = require$$0.$getRoot();
                                root.clear();
                                root.append(require$$0.$createParagraphNode());
                                root.selectEnd();
                                const nodes = clipboard.$generateNodesFromSerializedNodes(payload.nodes);
                                const sel = require$$0.$getSelection();
                                if (require$$0.$isRangeSelection(sel)) {
                                    clipboard.$insertGeneratedNodes(cellEditor, nodes, sel);
                                }
                            });
                            return;
                        }
                        // eslint-disable-next-line no-empty
                    }
                    catch (_c) { }
                }
                const htmlString = clipboardData !== null
                    ? yield getTypeFromObject(clipboardData, 'text/html')
                    : '';
                if (htmlString) {
                    try {
                        const parser = new DOMParser();
                        const dom = parser.parseFromString(htmlString, 'text/html');
                        const possibleTableElement = dom.querySelector('table');
                        if (possibleTableElement != null) {
                            const pasteRows = index.extractRowsFromHTML(possibleTableElement);
                            updateTableNode((tableNode) => {
                                const [x, y] = cellCoordMap.get(primarySelectedCellID);
                                require$$0.$addUpdateTag('history-push');
                                tableNode.mergeRows(x, y, pasteRows);
                            });
                            return;
                        }
                        $updateCells(rows, [primarySelectedCellID], cellCoordMap, cellEditor, updateTableNode, () => {
                            const root = require$$0.$getRoot();
                            root.clear();
                            root.append(require$$0.$createParagraphNode());
                            root.selectEnd();
                            const nodes = index.LexicalHtml_1.$generateNodesFromDOM(editor, dom);
                            const sel = require$$0.$getSelection();
                            if (require$$0.$isRangeSelection(sel)) {
                                clipboard.$insertGeneratedNodes(cellEditor, nodes, sel);
                            }
                        });
                        return;
                        // eslint-disable-next-line no-empty
                    }
                    catch (_d) { }
                }
                // Multi-line plain text in rich text mode pasted as separate paragraphs
                // instead of single paragraph with linebreaks.
                const text = clipboardData !== null
                    ? yield getTypeFromObject(clipboardData, 'text/plain')
                    : '';
                if (text != null) {
                    $updateCells(rows, [primarySelectedCellID], cellCoordMap, cellEditor, updateTableNode, () => {
                        const root = require$$0.$getRoot();
                        root.clear();
                        root.selectEnd();
                        const sel = require$$0.$getSelection();
                        if (sel !== null) {
                            sel.insertRawText(text);
                        }
                    });
                }
            }
        });
        const copyPrimaryCell = (event) => {
            if (primarySelectedCellID !== null && cellEditor !== null) {
                const cell = getCell(rows, primarySelectedCellID, cellCoordMap);
                const json = cell.json;
                const htmlString = index.cellHTMLCache.get(json) || null;
                if (htmlString === null) {
                    return;
                }
                const editorState = cellEditor.parseEditorState(json);
                const plainTextString = editorState.read(() => require$$0.$getRoot().getTextContent());
                const lexicalString = editorState.read(() => {
                    return JSON.stringify(clipboard.$generateJSONFromSelectedNodes(cellEditor, null));
                });
                copyDataToClipboard(event, htmlString, lexicalString, plainTextString);
            }
        };
        const copyCellRange = (event) => {
            const lastCellID = lastCellIDRef.current;
            if (primarySelectedCellID !== null &&
                cellEditor !== null &&
                lastCellID !== null) {
                const rect = getSelectedRect(primarySelectedCellID, lastCellID, cellCoordMap);
                if (rect === null) {
                    return;
                }
                const dom = index.exportTableCellsToHTML(rows, rect);
                const htmlString = dom.outerHTML;
                const plainTextString = dom.outerText;
                const tableNodeJSON = editor.getEditorState().read(() => {
                    const tableNode = require$$0.$getNodeByKey(nodeKey);
                    return tableNode.exportJSON();
                });
                tableNodeJSON.rows = extractCellsFromRows(rows, rect);
                const lexicalJSON = {
                    namespace: cellEditor._config.namespace,
                    nodes: [tableNodeJSON],
                };
                const lexicalString = JSON.stringify(lexicalJSON);
                copyDataToClipboard(event, htmlString, lexicalString, plainTextString);
            }
        };
        const handlePaste = (event, activeEditor) => {
            const selection = require$$0.$getSelection();
            if (primarySelectedCellID !== null &&
                !isEditing &&
                selection === null &&
                activeEditor === editor) {
                pasteContent(event);
                mouseDownRef.current = false;
                setSelectedCellIDs(NO_CELLS);
                return true;
            }
            return false;
        };
        const handleCopy = (event, activeEditor) => {
            const selection = require$$0.$getSelection();
            if (primarySelectedCellID !== null &&
                !isEditing &&
                selection === null &&
                activeEditor === editor) {
                if (selectedCellIDs.length === 0) {
                    copyPrimaryCell(event);
                }
                else {
                    copyCellRange(event);
                }
                return true;
            }
            return false;
        };
        return require$$4.mergeRegister(editor.registerCommand(require$$0.CLICK_COMMAND, (payload) => {
            const selection = require$$0.$getSelection();
            if (require$$0.$isNodeSelection(selection)) {
                return true;
            }
            return false;
        }, require$$0.COMMAND_PRIORITY_LOW), editor.registerCommand(require$$0.PASTE_COMMAND, handlePaste, require$$0.COMMAND_PRIORITY_LOW), editor.registerCommand(require$$0.COPY_COMMAND, handleCopy, require$$0.COMMAND_PRIORITY_LOW), editor.registerCommand(require$$0.CUT_COMMAND, (event, activeEditor) => {
            if (handleCopy(event, activeEditor)) {
                clearCellsCommand();
                return true;
            }
            return false;
        }, require$$0.COMMAND_PRIORITY_LOW), editor.registerCommand(require$$0.KEY_BACKSPACE_COMMAND, clearCellsCommand, require$$0.COMMAND_PRIORITY_LOW), editor.registerCommand(require$$0.KEY_DELETE_COMMAND, clearCellsCommand, require$$0.COMMAND_PRIORITY_LOW), editor.registerCommand(require$$0.FORMAT_TEXT_COMMAND, (payload) => {
            if (primarySelectedCellID !== null && !isEditing) {
                $updateCells(rows, [primarySelectedCellID, ...selectedCellIDs], cellCoordMap, cellEditor, updateTableNode, () => {
                    const sel = $createSelectAll();
                    sel.formatText(payload);
                });
                return true;
            }
            return false;
        }, require$$0.COMMAND_PRIORITY_LOW), editor.registerCommand(require$$0.KEY_ENTER_COMMAND, (event, targetEditor) => {
            const selection = require$$0.$getSelection();
            if (primarySelectedCellID === null &&
                !isEditing &&
                require$$0.$isNodeSelection(selection) &&
                selection.has(nodeKey) &&
                selection.getNodes().length === 1 &&
                targetEditor === editor) {
                const firstCellID = rows[0].cells[0].id;
                setPrimarySelectedCellID(firstCellID);
                focusCell(tableElem, firstCellID);
                event.preventDefault();
                event.stopPropagation();
                clearSelection();
                return true;
            }
            return false;
        }, require$$0.COMMAND_PRIORITY_LOW), editor.registerCommand(require$$0.KEY_TAB_COMMAND, (event) => {
            const selection = require$$0.$getSelection();
            if (!isEditing &&
                selection === null &&
                primarySelectedCellID !== null) {
                const isBackward = event.shiftKey;
                const [x, y] = cellCoordMap.get(primarySelectedCellID);
                event.preventDefault();
                let nextX = null;
                let nextY = null;
                if (x === 0 && isBackward) {
                    if (y !== 0) {
                        nextY = y - 1;
                        nextX = rows[nextY].cells.length - 1;
                    }
                }
                else if (x === rows[y].cells.length - 1 && !isBackward) {
                    if (y !== rows.length - 1) {
                        nextY = y + 1;
                        nextX = 0;
                    }
                }
                else if (!isBackward) {
                    nextX = x + 1;
                    nextY = y;
                }
                else {
                    nextX = x - 1;
                    nextY = y;
                }
                if (nextX !== null && nextY !== null) {
                    modifySelectedCells(nextX, nextY, false);
                    return true;
                }
            }
            return false;
        }, require$$0.COMMAND_PRIORITY_LOW), editor.registerCommand(require$$0.KEY_ARROW_UP_COMMAND, (event, targetEditor) => {
            const selection = require$$0.$getSelection();
            if (!isEditing && selection === null) {
                const extend = event.shiftKey;
                const cellID = extend
                    ? lastCellIDRef.current || primarySelectedCellID
                    : primarySelectedCellID;
                if (cellID !== null) {
                    const [x, y] = cellCoordMap.get(cellID);
                    if (y !== 0) {
                        modifySelectedCells(x, y - 1, extend);
                        return true;
                    }
                }
            }
            if (!require$$0.$isRangeSelection(selection) || targetEditor !== cellEditor) {
                return false;
            }
            if (selection.isCollapsed() &&
                selection.anchor
                    .getNode()
                    .getTopLevelElementOrThrow()
                    .getPreviousSibling() === null) {
                event.preventDefault();
                return true;
            }
            return false;
        }, require$$0.COMMAND_PRIORITY_LOW), editor.registerCommand(require$$0.KEY_ARROW_DOWN_COMMAND, (event, targetEditor) => {
            const selection = require$$0.$getSelection();
            if (!isEditing && selection === null) {
                const extend = event.shiftKey;
                const cellID = extend
                    ? lastCellIDRef.current || primarySelectedCellID
                    : primarySelectedCellID;
                if (cellID !== null) {
                    const [x, y] = cellCoordMap.get(cellID);
                    if (y !== rows.length - 1) {
                        modifySelectedCells(x, y + 1, extend);
                        return true;
                    }
                }
            }
            if (!require$$0.$isRangeSelection(selection) || targetEditor !== cellEditor) {
                return false;
            }
            if (selection.isCollapsed() &&
                selection.anchor
                    .getNode()
                    .getTopLevelElementOrThrow()
                    .getNextSibling() === null) {
                event.preventDefault();
                return true;
            }
            return false;
        }, require$$0.COMMAND_PRIORITY_LOW), editor.registerCommand(require$$0.KEY_ARROW_LEFT_COMMAND, (event, targetEditor) => {
            const selection = require$$0.$getSelection();
            if (!isEditing && selection === null) {
                const extend = event.shiftKey;
                const cellID = extend
                    ? lastCellIDRef.current || primarySelectedCellID
                    : primarySelectedCellID;
                if (cellID !== null) {
                    const [x, y] = cellCoordMap.get(cellID);
                    if (x !== 0) {
                        modifySelectedCells(x - 1, y, extend);
                        return true;
                    }
                }
            }
            if (!require$$0.$isRangeSelection(selection) || targetEditor !== cellEditor) {
                return false;
            }
            if (selection.isCollapsed() && selection.anchor.offset === 0) {
                event.preventDefault();
                return true;
            }
            return false;
        }, require$$0.COMMAND_PRIORITY_LOW), editor.registerCommand(require$$0.KEY_ARROW_RIGHT_COMMAND, (event, targetEditor) => {
            const selection = require$$0.$getSelection();
            if (!isEditing && selection === null) {
                const extend = event.shiftKey;
                const cellID = extend
                    ? lastCellIDRef.current || primarySelectedCellID
                    : primarySelectedCellID;
                if (cellID !== null) {
                    const [x, y] = cellCoordMap.get(cellID);
                    if (x !== rows[y].cells.length - 1) {
                        modifySelectedCells(x + 1, y, extend);
                        return true;
                    }
                }
            }
            if (!require$$0.$isRangeSelection(selection) || targetEditor !== cellEditor) {
                return false;
            }
            if (selection.isCollapsed()) {
                const anchor = selection.anchor;
                if ((anchor.type === 'text' &&
                    anchor.offset === anchor.getNode().getTextContentSize()) ||
                    (anchor.type === 'element' &&
                        anchor.offset === anchor.getNode().getChildrenSize())) {
                    event.preventDefault();
                    return true;
                }
            }
            return false;
        }, require$$0.COMMAND_PRIORITY_LOW), editor.registerCommand(require$$0.KEY_ESCAPE_COMMAND, (event, targetEditor) => {
            const selection = require$$0.$getSelection();
            if (!isEditing && selection === null && targetEditor === editor) {
                setSelected(true);
                setPrimarySelectedCellID(null);
                selectTable();
                return true;
            }
            if (!require$$0.$isRangeSelection(selection)) {
                return false;
            }
            if (isEditing) {
                saveEditorToJSON();
                setIsEditing(false);
                if (primarySelectedCellID !== null) {
                    setTimeout(() => {
                        focusCell(tableElem, primarySelectedCellID);
                    }, 20);
                }
                return true;
            }
            return false;
        }, require$$0.COMMAND_PRIORITY_LOW));
    }, [
        cellCoordMap,
        cellEditor,
        clearCellsCommand,
        clearSelection,
        editor,
        isEditing,
        modifySelectedCells,
        nodeKey,
        primarySelectedCellID,
        rows,
        saveEditorToJSON,
        selectTable,
        selectedCellIDs,
        setSelected,
        updateTableNode,
    ]);
    if (cellEditor === null) {
        return;
    }
    return (jsxRuntime.jsxs("div", { style: { position: 'relative' }, children: [jsxRuntime.jsx("table", { className: `${theme.table} ${isSelected ? theme.tableSelected : ''}`, ref: tableRef, tabIndex: -1, children: jsxRuntime.jsx("tbody", { children: rows.map((row) => (jsxRuntime.jsx("tr", { className: theme.tableRow, children: row.cells.map((cell) => {
                            const { id } = cell;
                            return (jsxRuntime.jsx(TableCell, { cell: cell, theme: theme, isSelected: selectedCellSet.has(id), isPrimarySelected: primarySelectedCellID === id, isEditing: isEditing, sortingOptions: sortingOptions, cellEditor: cellEditor, updateCellsByID: updateCellsByID, updateTableNode: updateTableNode, cellCoordMap: cellCoordMap, rows: rows, setSortingOptions: setSortingOptions }, id));
                        }) }, row.id))) }) }), showAddColumns && (jsxRuntime.jsx("button", { className: theme.tableAddColumns, onClick: addColumns })), showAddRows && (jsxRuntime.jsx("button", { className: theme.tableAddRows, onClick: addRows, ref: addRowsRef })), resizingID !== null && (jsxRuntime.jsx("div", { className: theme.tableResizeRuler, ref: tableResizerRulerRef }))] }));
}

exports["default"] = TableComponent;
