/// <reference types="react" />
export type EditorOptions = {
    /**
     * 编辑器内容 placeholder
     */
    Placeholder?: () => JSX.Element;
    /**
     * 编辑器标题 placeholder
     */
    titlePlaceHolder?: string;
    /**
     * 编辑器初始 html
     */
    initialHtml?: string;
    /**
     * 编辑器发生修改事件
     * @param data
     * @returns
     */
    onChange: (data: {
        title: string;
        html: string;
    }) => void;
    /**
     * 图片选择器的图片来源
     *
     * 如果不提供图片来源,默认使用输入 url 的方式来添加图片
     */
    getImages?: PageQuery<string>;
    /**
     * 图片上传路径
     *
     * 如果不提供，不能上传图片
     */
    uploadImageUrl?: string;
};
export type PostResult = {
    id: number;
    title: string;
    author: string;
    version: string;
    content: string;
    hash: string;
    tags: string[];
    published: boolean;
    createdAt: number;
    updatedAt: number;
};
export type ListResults<T> = {
    count: number;
    results: T[];
};
export type HistoryResult = {
    id: number;
    postId: number;
    origin: string;
    hash: string;
    changeset: ChangeSet[];
    createdAt: number;
    updatedAt: number;
};
export type ChangeSet = {
    kind: string;
    text: string;
};
export type ImgFile = {
    id: number;
    originPath: string;
    minPath: string;
    originSize: number;
    minSize: number;
    owner: string;
    description: string;
    tags: string[];
    createdAt: number;
    updatedAt: number;
};
export type PageQuery<T> = ({ limit, offset }: {
    limit: number;
    offset: number;
}) => Promise<ListResults<T>>;
